Method: Sidekiq::Job::Iterable#each_iteration

Defined in:
lib/sidekiq/job/iterable.rb

#each_iterationvoid

This method returns an undefined value.

The action to be performed on each item from the enumerator.

Raises:

  • (NotImplementedError)

    with a message advising subclasses to implement an override for this method.



132
133
134
# File 'lib/sidekiq/job/iterable.rb', line 132

def each_iteration(*)
  raise NotImplementedError, "#{self.class.name} must implement an '#each_iteration' method"
end