Class: Sidekiq::Launcher

Inherits:
Object
  • Object
show all
Includes:
Component
Defined in:
lib/sidekiq/launcher.rb

Overview

The Launcher starts the Manager and Poller threads and provides the process heartbeat.

Constant Summary collapse

STATS_TTL =

5 years

5 * 365 * 24 * 60 * 60
PROCTITLES =
[
  proc { "sidekiq" },
  proc { Sidekiq::VERSION },
  proc { |me, data| data["tag"] },
  proc { |me, data| "[#{Processor::WORK_STATE.size} of #{data["concurrency"]} busy]" },
  proc { |me, data| "stopping" if me.stopping? }
]
BEAT_PAUSE =
5
RTT_READINGS =

We run the heartbeat every five seconds. Capture five samples of RTT, log a warning if each sample is above our warning threshold.

RingBuffer.new(5)
RTT_WARNING_LEVEL =
50_000
MEMORY_GRABBER =
case RUBY_PLATFORM
when /linux/
  ->(pid) {
    IO.readlines("/proc/#{$$}/status").each do |line|
      next unless line.start_with?("VmRSS:")
      break line.split[1].to_i
    end
  }
when /darwin|bsd/
  ->(pid) {
    `ps -o pid,rss -p #{pid}`.lines.last.split.last.to_i
  }
else
  ->(pid) { 0 }
end

Instance Attribute Summary collapse

Attributes included from Component

#config

Class Method Summary collapse

Instance Method Summary collapse

Methods included from Component

#fire_event, #handle_exception, #hostname, #identity, #logger, #process_nonce, #redis, #safe_thread, #tid, #watchdog

Constructor Details

#initialize(options) ⇒ Launcher

Returns a new instance of Launcher.



25
26
27
28
29
30
31
# File 'lib/sidekiq/launcher.rb', line 25

def initialize(options)
  @config = options
  options[:fetch] ||= BasicFetch.new(options)
  @manager = Sidekiq::Manager.new(options)
  @poller = Sidekiq::Scheduled::Poller.new(options)
  @done = false
end

Instance Attribute Details

#fetcherObject

Returns the value of attribute fetcher.



23
24
25
# File 'lib/sidekiq/launcher.rb', line 23

def fetcher
  @fetcher
end

#managerObject

Returns the value of attribute manager.



23
24
25
# File 'lib/sidekiq/launcher.rb', line 23

def manager
  @manager
end

#pollerObject

Returns the value of attribute poller.



23
24
25
# File 'lib/sidekiq/launcher.rb', line 23

def poller
  @poller
end

Class Method Details

.flush_statsObject



101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
# File 'lib/sidekiq/launcher.rb', line 101

def self.flush_stats
  fails = Processor::FAILURE.reset
  procd = Processor::PROCESSED.reset
  return if fails + procd == 0

  nowdate = Time.now.utc.strftime("%Y-%m-%d")
  begin
    Sidekiq.redis do |conn|
      conn.pipelined do |pipeline|
        pipeline.incrby("stat:processed", procd)
        pipeline.incrby("stat:processed:#{nowdate}", procd)
        pipeline.expire("stat:processed:#{nowdate}", STATS_TTL)

        pipeline.incrby("stat:failed", fails)
        pipeline.incrby("stat:failed:#{nowdate}", fails)
        pipeline.expire("stat:failed:#{nowdate}", STATS_TTL)
      end
    end
  rescue => ex
    # we're exiting the process, things might be shut down so don't
    # try to handle the exception
    Sidekiq.logger.warn("Unable to flush stats: #{ex}")
  end
end

Instance Method Details

#check_rttObject



201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
# File 'lib/sidekiq/launcher.rb', line 201

def check_rtt
  a = b = 0
  redis do |x|
    a = ::Process.clock_gettime(::Process::CLOCK_MONOTONIC, :microsecond)
    x.ping
    b = ::Process.clock_gettime(::Process::CLOCK_MONOTONIC, :microsecond)
  end
  rtt = b - a
  RTT_READINGS << rtt
  # Ideal RTT for Redis is < 1000µs
  # Workable is < 10,000µs
  # Log a warning if it's a disaster.
  if RTT_READINGS.all? { |x| x > RTT_WARNING_LEVEL }
    logger.warn <<~EOM
      Your Redis network connection is performing extremely poorly.
      Last RTT readings were #{RTT_READINGS.buffer.inspect}, ideally these should be < 1000.
      Ensure Redis is running in the same AZ or datacenter as Sidekiq.
      If these values are close to 100,000, that means your Sidekiq process may be
      CPU-saturated; reduce your concurrency and/or see https://github.com/mperham/sidekiq/discussions/5039
    EOM
    RTT_READINGS.reset
  end
  rtt
end

#clear_heartbeatObject



81
82
83
84
85
86
87
88
89
90
91
92
93
# File 'lib/sidekiq/launcher.rb', line 81

def clear_heartbeat
  # Remove record from Redis since we are shutting down.
  # Note we don't stop the heartbeat thread; if the process
  # doesn't actually exit, it'll reappear in the Web UI.
  redis do |conn|
    conn.pipelined do |pipeline|
      pipeline.srem("processes", identity)
      pipeline.unlink("#{identity}:work")
    end
  end
rescue
  # best effort, ignore network errors
end

#heartbeatObject



95
96
97
98
99
# File 'lib/sidekiq/launcher.rb', line 95

def heartbeat
  $0 = PROCTITLES.map { |proc| proc.call(self, to_data) }.compact.join(" ")

  
end

#memory_usage(pid) ⇒ Object



242
243
244
# File 'lib/sidekiq/launcher.rb', line 242

def memory_usage(pid)
  MEMORY_GRABBER.call(pid)
end

#quietObject

Stops this instance from processing any more jobs,



41
42
43
44
45
# File 'lib/sidekiq/launcher.rb', line 41

def quiet
  @done = true
  @manager.quiet
  @poller.terminate
end

#runObject



33
34
35
36
37
# File 'lib/sidekiq/launcher.rb', line 33

def run
  @thread = safe_thread("heartbeat", &method(:start_heartbeat))
  @poller.start
  @manager.start
end

#start_heartbeatObject



73
74
75
76
77
78
79
# File 'lib/sidekiq/launcher.rb', line 73

def start_heartbeat
  loop do
    heartbeat
    sleep BEAT_PAUSE
  end
  logger.info("Heartbeat stopping...")
end

#stopObject

Shuts down this Sidekiq instance. Waits up to the deadline for all jobs to complete.



48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
# File 'lib/sidekiq/launcher.rb', line 48

def stop
  deadline = ::Process.clock_gettime(::Process::CLOCK_MONOTONIC) + @config[:timeout]

  @done = true
  @manager.quiet
  @poller.terminate

  @manager.stop(deadline)

  # Requeue everything in case there was a thread which fetched a job while the process was stopped.
  # This call is a no-op in Sidekiq but necessary for Sidekiq Pro.
  strategy = @config[:fetch]
  strategy.bulk_requeue([], @config)

  clear_heartbeat
end

#stopping?Boolean

Returns:

  • (Boolean)


65
66
67
# File 'lib/sidekiq/launcher.rb', line 65

def stopping?
  @done
end

#to_dataObject



246
247
248
249
250
251
252
253
254
255
256
257
# File 'lib/sidekiq/launcher.rb', line 246

def to_data
  @data ||= {
    "hostname" => hostname,
    "started_at" => Time.now.to_f,
    "pid" => ::Process.pid,
    "tag" => @config[:tag] || "",
    "concurrency" => @config[:concurrency],
    "queues" => @config[:queues].uniq,
    "labels" => @config[:labels],
    "identity" => identity
  }
end

#to_jsonObject



259
260
261
262
263
# File 'lib/sidekiq/launcher.rb', line 259

def to_json
  # this data changes infrequently so dump it to a string
  # now so we don't need to dump it every heartbeat.
  @json ||= Sidekiq.dump_json(to_data)
end

#Object



127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
# File 'lib/sidekiq/launcher.rb', line 127

def 
  key = identity
  fails = procd = 0

  begin
    fails = Processor::FAILURE.reset
    procd = Processor::PROCESSED.reset
    curstate = Processor::WORK_STATE.dup

    nowdate = Time.now.utc.strftime("%Y-%m-%d")

    redis do |conn|
      conn.multi do |transaction|
        transaction.incrby("stat:processed", procd)
        transaction.incrby("stat:processed:#{nowdate}", procd)
        transaction.expire("stat:processed:#{nowdate}", STATS_TTL)

        transaction.incrby("stat:failed", fails)
        transaction.incrby("stat:failed:#{nowdate}", fails)
        transaction.expire("stat:failed:#{nowdate}", STATS_TTL)
      end

      # work is the current set of executing jobs
      work_key = "#{key}:work"
      conn.pipelined do |transaction|
        transaction.unlink(work_key)
        curstate.each_pair do |tid, hash|
          transaction.hset(work_key, tid, Sidekiq.dump_json(hash))
        end
        transaction.expire(work_key, 60)
      end
    end

    rtt = check_rtt

    fails = procd = 0
    kb = memory_usage(::Process.pid)

    _, exists, _, _, msg = redis { |conn|
      conn.multi { |transaction|
        transaction.sadd("processes", key)
        transaction.exists?(key)
        transaction.hmset(key, "info", to_json,
          "busy", curstate.size,
          "beat", Time.now.to_f,
          "rtt_us", rtt,
          "quiet", @done.to_s,
          "rss", kb)
        transaction.expire(key, 60)
        transaction.rpop("#{key}-signals")
      }
    }

    # first heartbeat or recovering from an outage and need to reestablish our heartbeat
    fire_event(:heartbeat) unless exists

    return unless msg

    ::Process.kill(msg, ::Process.pid)
  rescue => e
    # ignore all redis/network issues
    logger.error("heartbeat: #{e}")
    # don't lose the counts if there was a network issue
    Processor::PROCESSED.incr(procd)
    Processor::FAILURE.incr(fails)
  end
end