Method: Sidekiq::Process#quiet!
- Defined in:
- lib/sidekiq/api.rb
#quiet! ⇒ Object
Signal this process to stop processing new jobs. It will continue to execute jobs it has already fetched. This method is asynchronous and it can take 5-10 seconds for the process to quiet.
1174 1175 1176 1177 1178 |
# File 'lib/sidekiq/api.rb', line 1174 def quiet! raise "Can't quiet an embedded process" if signal("TSTP") end |