Class: Sidekiq::Process

Inherits:
Object
  • Object
show all
Defined in:
lib/sidekiq/api.rb

Overview

Sidekiq::Process represents an active Sidekiq process talking with Redis. Each process has a set of attributes which look like this:

'hostname' => 'app-1.example.com',
'started_at' => <process start time>,
'pid' => 12345,
'tag' => 'myapp'
'concurrency' => 5,
'capsules' => {"default" => {"mode" => "weighted", "concurrency" => 5, "weights" => {"default" => 2, "low" => 1}},
'busy' => 3,
'beat' => <last heartbeat>,
'identity' => <unique string identifying the process>,
'embedded' => true,

}

Instance Method Summary collapse

Constructor Details

#initialize(hash) ⇒ Process

This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.

:nodoc:



1121
1122
1123
# File 'lib/sidekiq/api.rb', line 1121

def initialize(hash)
  @attribs = hash
end

Instance Method Details

#[](key) ⇒ Object



1133
1134
1135
# File 'lib/sidekiq/api.rb', line 1133

def [](key)
  @attribs[key]
end

#capsulesObject



1169
1170
1171
# File 'lib/sidekiq/api.rb', line 1169

def capsules
  self["capsules"]
end

#dump_threadsObject

Signal this process to log backtraces for all threads. Useful if you have a frozen or deadlocked process which is still sending a heartbeat. This method is asynchronous and it can take 5-10 seconds.



1205
1206
1207
# File 'lib/sidekiq/api.rb', line 1205

def dump_threads
  signal("TTIN")
end

#embedded?Boolean

Returns:

  • (Boolean)


1177
1178
1179
# File 'lib/sidekiq/api.rb', line 1177

def embedded?
  self["embedded"]
end

#identityObject Also known as: id



1137
1138
1139
# File 'lib/sidekiq/api.rb', line 1137

def identity
  self["identity"]
end

#labelsObject



1129
1130
1131
# File 'lib/sidekiq/api.rb', line 1129

def labels
  self["labels"].to_a
end

#leader?Boolean

Returns:

  • (Boolean)


1214
1215
1216
# File 'lib/sidekiq/api.rb', line 1214

def leader?
  Sidekiq.redis { |c| c.get("dear-leader") == identity }
end

#queuesObject

deprecated, use capsules below



1143
1144
1145
1146
1147
1148
1149
1150
# File 'lib/sidekiq/api.rb', line 1143

def queues
  # Backwards compatibility with <8.0.8
  if !self["capsules"]
    self["queues"]
  else
    capsules.values.flat_map { |x| x["weights"].keys }.uniq
  end
end

#quiet!Object

Signal this process to stop processing new jobs. It will continue to execute jobs it has already fetched. This method is asynchronous and it can take 5-10 seconds for the process to quiet.



1185
1186
1187
1188
1189
# File 'lib/sidekiq/api.rb', line 1185

def quiet!
  raise "Can't quiet an embedded process" if embedded?

  signal("TSTP")
end

#stop!Object

Signal this process to shutdown. It will shutdown within its configured :timeout value, default 25 seconds. This method is asynchronous and it can take 5-10 seconds for the process to start shutting down.



1195
1196
1197
1198
1199
# File 'lib/sidekiq/api.rb', line 1195

def stop!
  raise "Can't stop an embedded process" if embedded?

  signal("TERM")
end

#stopping?Boolean

Returns true if this process is quiet or shutting down.

Returns:

  • (Boolean)

    true if this process is quiet or shutting down



1210
1211
1212
# File 'lib/sidekiq/api.rb', line 1210

def stopping?
  self["quiet"] == "true"
end

#tagObject



1125
1126
1127
# File 'lib/sidekiq/api.rb', line 1125

def tag
  self["tag"]
end

#versionObject



1173
1174
1175
# File 'lib/sidekiq/api.rb', line 1173

def version
  self["version"]
end

#weightsObject

deprecated, use capsules below



1153
1154
1155
1156
1157
1158
1159
1160
1161
1162
1163
1164
1165
1166
1167
# File 'lib/sidekiq/api.rb', line 1153

def weights
  # Backwards compatibility with <8.0.8
  if !self["capsules"]
    self["weights"]
  else
    hash = {}
    capsules.values.each do |cap|
      # Note: will lose data if two capsules are processing the same named queue
      cap["weights"].each_pair do |queue, weight|
        hash[queue] = weight
      end
    end
    hash
  end
end