Class: Sidekiq::ProcessSet

Inherits:
Object
  • Object
show all
Includes:
Enumerable
Defined in:
lib/sidekiq/api.rb

Overview

Enumerates the set of Sidekiq processes which are actively working right now. Each process sends a heartbeat to Redis every 5 seconds so this set should be relatively accurate, barring network partitions.

Yields a Sidekiq::Process.

Instance Method Summary collapse

Constructor Details

#initialize(clean_plz = true) ⇒ ProcessSet

Returns a new instance of ProcessSet.



788
789
790
# File 'lib/sidekiq/api.rb', line 788

def initialize(clean_plz = true)
  cleanup if clean_plz
end

Instance Method Details

#cleanupObject

Cleans up dead processes recorded in Redis. Returns the number of processes cleaned.



794
795
796
797
798
799
800
801
802
803
804
805
806
807
808
809
810
811
812
813
# File 'lib/sidekiq/api.rb', line 794

def cleanup
  count = 0
  Sidekiq.redis do |conn|
    procs = conn.sscan_each("processes").to_a.sort
    heartbeats = conn.pipelined { |pipeline|
      procs.each do |key|
        pipeline.hget(key, "info")
      end
    }

    # the hash named key has an expiry of 60 seconds.
    # if it's not found, that means the process has not reported
    # in to Redis and probably died.
    to_prune = procs.select.with_index { |proc, i|
      heartbeats[i].nil?
    }
    count = conn.srem("processes", to_prune) unless to_prune.empty?
  end
  count
end

#eachObject



815
816
817
818
819
820
821
822
823
824
825
826
827
828
829
830
831
832
833
834
835
836
837
838
839
840
841
842
# File 'lib/sidekiq/api.rb', line 815

def each
  result = Sidekiq.redis { |conn|
    procs = conn.sscan_each("processes").to_a.sort

    # We're making a tradeoff here between consuming more memory instead of
    # making more roundtrips to Redis, but if you have hundreds or thousands of workers,
    # you'll be happier this way
    conn.pipelined do |pipeline|
      procs.each do |key|
        pipeline.hmget(key, "info", "busy", "beat", "quiet", "rss", "rtt_us")
      end
    end
  }

  result.each do |info, busy, at_s, quiet, rss, rtt|
    # If a process is stopped between when we query Redis for `procs` and
    # when we query for `result`, we will have an item in `result` that is
    # composed of `nil` values.
    next if info.nil?

    hash = Sidekiq.load_json(info)
    yield Process.new(hash.merge("busy" => busy.to_i,
      "beat" => at_s.to_f,
      "quiet" => quiet,
      "rss" => rss.to_i,
      "rtt_us" => rtt.to_i))
  end
end

#leaderObject

Returns the identity of the current cluster leader or “” if no leader. This is a Sidekiq Enterprise feature, will always return “” in Sidekiq or Sidekiq Pro.



867
868
869
870
871
872
873
874
# File 'lib/sidekiq/api.rb', line 867

def leader
  @leader ||= begin
    x = Sidekiq.redis { |c| c.get("dear-leader") }
    # need a non-falsy value so we can memoize
    x ||= ""
    x
  end
end

#sizeObject

This method is not guaranteed accurate since it does not prune the set based on current heartbeat. #each does that and ensures the set only contains Sidekiq processes which have sent a heartbeat within the last 60 seconds.



848
849
850
# File 'lib/sidekiq/api.rb', line 848

def size
  Sidekiq.redis { |conn| conn.scard("processes") }
end

#total_concurrencyObject

Total number of threads available to execute jobs. For Sidekiq Enterprise customers this number (in production) must be less than or equal to your licensed concurrency.



855
856
857
# File 'lib/sidekiq/api.rb', line 855

def total_concurrency
  sum { |x| x["concurrency"].to_i }
end

#total_rss_in_kbObject Also known as: total_rss



859
860
861
# File 'lib/sidekiq/api.rb', line 859

def total_rss_in_kb
  sum { |x| x["rss"].to_i }
end