Module: Sidekiq::WebHelpers

Defined in:
lib/sidekiq/web/helpers.rb

Overview

These methods are available to pages within the Web UI and UI extensions. They are not public APIs for applications to use.

Constant Summary collapse

SAFE_QPARAMS =
%w[page direction]
RETRY_JOB_KEYS =
Set.new(%w[
  queue class args retry_count retried_at failed_at
  jid error_message error_class backtrace
  error_backtrace enqueued_at retry wrapped
  created_at tags display_class
])

Instance Method Summary collapse

Instance Method Details

#add_to_headObject

This view helper provide ability display you html code in to head of page. Example:

<% add_to_head do %>
  <link rel="stylesheet" .../>
  <meta .../>
<% end %>


180
181
182
183
# File 'lib/sidekiq/web/helpers.rb', line 180

def add_to_head
  @head_html ||= []
  @head_html << yield.dup if block_given?
end

#available_localesObject



135
136
137
# File 'lib/sidekiq/web/helpers.rb', line 135

def available_locales
  @@available_locales ||= Set.new(locale_files.map { |path| File.basename(path, ".yml") })
end

#clear_cachesObject



123
124
125
126
127
# File 'lib/sidekiq/web/helpers.rb', line 123

def clear_caches
  @@strings = nil
  @@locale_files = nil
  @@available_locales = nil
end

#csp_nonceObject



375
376
377
# File 'lib/sidekiq/web/helpers.rb', line 375

def csp_nonce
  env[:csp_nonce]
end

#csrf_tagObject



371
372
373
# File 'lib/sidekiq/web/helpers.rb', line 371

def csrf_tag
  ""
end

#current_pathObject



306
307
308
# File 'lib/sidekiq/web/helpers.rb', line 306

def current_path
  @current_path ||= request.path_info.gsub(/^\//, "")
end

#current_statusObject



310
311
312
# File 'lib/sidekiq/web/helpers.rb', line 310

def current_status
  (workset.size == 0) ? "idle" : "active"
end

#delete_or_add_queue(job, params) ⇒ Object



470
471
472
473
474
475
476
# File 'lib/sidekiq/web/helpers.rb', line 470

def delete_or_add_queue(job, params)
  if params["delete"]
    job.delete
  elsif params["add_to_queue"]
    job.add_to_queue
  end
end

#display_args(args, truncate_after_chars = 2000) ⇒ Object



358
359
360
361
362
363
364
365
366
367
368
369
# File 'lib/sidekiq/web/helpers.rb', line 358

def display_args(args, truncate_after_chars = 2000)
  return "Invalid job payload, args is nil" if args.nil?
  return "Invalid job payload, args must be an Array, not #{args.class.name}" unless args.is_a?(Array)

  begin
    args.map { |arg|
      h(truncate(to_display(arg), truncate_after_chars))
    }.join(", ")
  rescue
    "Illegal job arguments: #{h args.inspect}"
  end
end

#display_custom_headObject



185
186
187
# File 'lib/sidekiq/web/helpers.rb', line 185

def display_custom_head
  @head_html.join if defined?(@head_html)
end

#display_tags(job, within = "retries") ⇒ Object



166
167
168
169
170
# File 'lib/sidekiq/web/helpers.rb', line 166

def display_tags(job, within = "retries")
  job.tags.map { |tag|
    "<span class='label label-info jobtag jobtag-#{Rack::Utils.escape_html(tag)}'>#{filter_link(tag, within)}</span>"
  }.join(" ")
end

#environment_title_prefixObject



441
442
443
444
445
# File 'lib/sidekiq/web/helpers.rb', line 441

def environment_title_prefix
  environment = Sidekiq.default_configuration[:environment] || ENV["APP_ENV"] || ENV["RAILS_ENV"] || ENV["RACK_ENV"] || "development"

  "[#{environment.upcase}] " unless environment == "production"
end


158
159
160
161
162
163
164
# File 'lib/sidekiq/web/helpers.rb', line 158

def filter_link(jid, within = "retries")
  if within.nil?
    ::Rack::Utils.escape_html(jid)
  else
    "<a href='#{root_path}#{within}?substr=#{jid}'>#{::Rack::Utils.escape_html(jid)}</a>"
  end
end

#filtering(which, placeholder_key: "AnyJobContent", label_key: "Filter") ⇒ Object



154
155
156
# File 'lib/sidekiq/web/helpers.rb', line 154

def filtering(which, placeholder_key: "AnyJobContent", label_key: "Filter")
  erb(:filtering, locals: {which:, placeholder_key:, label_key:})
end

#find_locale_files(lang) ⇒ Object



139
140
141
# File 'lib/sidekiq/web/helpers.rb', line 139

def find_locale_files(lang)
  locale_files.select { |file| file =~ /\/#{lang}\.yml$/ }
end

#format_memory(rss_kb) ⇒ Object



404
405
406
407
408
409
410
411
412
413
414
# File 'lib/sidekiq/web/helpers.rb', line 404

def format_memory(rss_kb)
  return "0" if rss_kb.nil? || rss_kb == 0

  if rss_kb < 100_000
    "#{number_with_delimiter(rss_kb)} KB"
  elsif rss_kb < 10_000_000
    "#{number_with_delimiter((rss_kb / 1024.0).to_i)} MB"
  else
    "#{number_with_delimiter(rss_kb / (1024.0 * 1024.0), precision: 1)} GB"
  end
end

#get_localeObject



250
251
252
# File 'lib/sidekiq/web/helpers.rb', line 250

def get_locale
  strings(locale)
end

#h(text) ⇒ Object



421
422
423
424
425
426
427
# File 'lib/sidekiq/web/helpers.rb', line 421

def h(text)
  ::Rack::Utils.escape_html(text.to_s)
rescue ArgumentError => e
  raise unless e.message.eql?("invalid byte sequence in UTF-8")
  text.encode!("UTF-16", "UTF-8", invalid: :replace, replace: "").encode!("UTF-8", "UTF-16")
  retry
end

#job_params(job, score) ⇒ Object



330
331
332
# File 'lib/sidekiq/web/helpers.rb', line 330

def job_params(job, score)
  "#{score}-#{job["jid"]}"
end

#language_name(locale) ⇒ Object



143
144
145
# File 'lib/sidekiq/web/helpers.rb', line 143

def language_name(locale)
  strings(locale).fetch("LanguageName", locale)
end

#localeObject

Given an Accept-Language header like “fr-FR,fr;q=0.8,en-US;q=0.6,en;q=0.4,ru;q=0.2” this method will try to best match the available locales to the user’s preferred languages.



215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
# File 'lib/sidekiq/web/helpers.rb', line 215

def locale
  # session[:locale] is set via the locale selector from the footer
  @locale ||= if (l = session&.fetch(:locale, nil)) && available_locales.include?(l)
    l
  else
    matched_locale = nil
    # Attempt to find a case-insensitive exact match first
    user_preferred_languages.each do |preferred|
      # We only care about the language and primary subtag
      # "en-GB-oxendict" becomes "en-GB"
      language_tag = preferred.split("-")[0..1].join("-")
      matched_locale = available_locales.find { |available_locale| available_locale.casecmp?(language_tag) }
      break if matched_locale
    end

    return matched_locale if matched_locale

    # Find the first base language match
    # "en-US,es-MX;q=0.9" matches "en"
    user_preferred_languages.each do |preferred|
      base_language = preferred.split("-", 2).first
      matched_locale = available_locales.find { |available_locale| available_locale.casecmp?(base_language) }
      break if matched_locale
    end

    matched_locale || "en"
  end
end

#locale_filesObject



129
130
131
132
133
# File 'lib/sidekiq/web/helpers.rb', line 129

def locale_files
  @@locale_files ||= config.locales.flat_map { |path|
    Dir["#{path}/*.yml"]
  }
end

#number_with_delimiter(number, options = {}) ⇒ Object



416
417
418
419
# File 'lib/sidekiq/web/helpers.rb', line 416

def number_with_delimiter(number, options = {})
  precision = options[:precision] || 0
  %(<span data-nwp="#{precision}">#{number.round(precision)}</span>)
end

#parse_key(key) ⇒ Object



334
335
336
337
# File 'lib/sidekiq/web/helpers.rb', line 334

def parse_key(key)
  score, jid = key.split("-", 2)
  [score.to_f, jid]
end

#parse_yaml_new(path) ⇒ Object



86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/sidekiq/web/helpers.rb', line 86

def parse_yaml_new(path)
  locale = nil
  map = {}
  IO.readlines(path, chomp: true).each do |line|
    case line
    when /\A\s*\#.*/
      # line comment
    when !locale && /\A([a-zA-Z\-_]+):/
      locale = $1
      map[locale] = {}
    when /\A\s+(\w+):\s+(.+)\z/
      # A few values have double quotes to include special characters in YAML.
      # Strip them off manually as our greedy match will include them.
      key = $1
      s = $2
      s = s[1..] if s[0] == "\""
      s = s[0..-2] if s[-1] == "\""
      map[locale][key] = s
    else
      raise ArgumentError, "unable to parse #{path}: #{line}"
    end
  end
  map
end

#parse_yaml_old(path) ⇒ Object



81
82
83
84
# File 'lib/sidekiq/web/helpers.rb', line 81

def parse_yaml_old(path)
  require "yaml"
  YAML.safe_load_file(path)
end

#pollable?Boolean

Returns:

  • (Boolean)


455
456
457
458
# File 'lib/sidekiq/web/helpers.rb', line 455

def pollable?
  # there's no point to refreshing the metrics pages every N seconds
  !(current_path == "" || current_path.index("metrics"))
end

#processesObject



271
272
273
# File 'lib/sidekiq/web/helpers.rb', line 271

def processes
  @processes ||= Sidekiq::ProcessSet.new
end

#product_versionObject



447
448
449
# File 'lib/sidekiq/web/helpers.rb', line 447

def product_version
  "Sidekiq v#{Sidekiq::VERSION}"
end

#qparams(options) ⇒ Object

Merge options with current params, filter safe params, and stringify to query string



342
343
344
345
346
# File 'lib/sidekiq/web/helpers.rb', line 342

def qparams(options)
  stringified_options = options.transform_keys(&:to_s)

  to_query_string(request.params.merge(stringified_options))
end

#queue_names_by_capsule(pro) ⇒ Object



319
320
321
322
323
324
325
326
327
328
# File 'lib/sidekiq/web/helpers.rb', line 319

def queue_names_by_capsule(pro)
  cap = pro.capsules
  if cap
    cap.map { |k, v| v["weights"].keys.join(", ") }.join("; ")
  else
    # DEPRECATED Backwards compatibility with older processes.
    # 'capsules' element added in v8.0.9
    pro.queues.join(", ")
  end
end

#redirect_with_query(url) ⇒ Object

Any paginated list that performs an action needs to redirect back to the proper page after performing that action.



431
432
433
434
435
436
437
438
439
# File 'lib/sidekiq/web/helpers.rb', line 431

def redirect_with_query(url)
  r = request.referer
  if r && r =~ /\?/
    ref = URI(r)
    redirect("#{url}?#{ref.query}")
  else
    redirect url
  end
end

#redis_infoObject



298
299
300
# File 'lib/sidekiq/web/helpers.rb', line 298

def redis_info
  @info ||= Sidekiq.default_configuration.redis_info
end

#redis_urlObject



292
293
294
295
296
# File 'lib/sidekiq/web/helpers.rb', line 292

def redis_url
  Sidekiq.redis do |conn|
    conn.config.server_url
  end
end

#relative_time(time) ⇒ Object



314
315
316
317
# File 'lib/sidekiq/web/helpers.rb', line 314

def relative_time(time)
  stamp = time.getutc.iso8601
  %(<time class="ltr" dir="ltr" title="#{stamp}" datetime="#{stamp}">#{time}</time>)
end

#retry_extra_items(retry_job) ⇒ Object



396
397
398
399
400
401
402
# File 'lib/sidekiq/web/helpers.rb', line 396

def retry_extra_items(retry_job)
  @retry_extra_items ||= {}.tap do |extra|
    retry_job.item.each do |key, value|
      extra[key] = value unless RETRY_JOB_KEYS.include?(key)
    end
  end
end

#retry_or_delete_or_kill(job, params) ⇒ Object



460
461
462
463
464
465
466
467
468
# File 'lib/sidekiq/web/helpers.rb', line 460

def retry_or_delete_or_kill(job, params)
  if params["retry"]
    job.retry
  elsif params["delete"]
    job.delete
  elsif params["kill"]
    job.kill
  end
end

#root_pathObject



302
303
304
# File 'lib/sidekiq/web/helpers.rb', line 302

def root_path
  "#{env["SCRIPT_NAME"]}/"
end

#rtl?Boolean

Returns:

  • (Boolean)


193
194
195
# File 'lib/sidekiq/web/helpers.rb', line 193

def rtl?
  text_direction == "rtl"
end

#script_tag(location, **kwargs) ⇒ Object



39
40
41
42
43
44
45
46
47
48
# File 'lib/sidekiq/web/helpers.rb', line 39

def script_tag(location, **kwargs)
  global = location.match?(/:\/\//)
  location = root_path + location if !global && !location.start_with?(root_path)
  attrs = {
    type: "text/javascript",
    nonce: csp_nonce,
    src: location
  }
  html_tag(:script, attrs.merge(kwargs)) {}
end

#search(jobset, substr) ⇒ Object



147
148
149
150
151
152
# File 'lib/sidekiq/web/helpers.rb', line 147

def search(jobset, substr)
  resultset = jobset.scan(substr).to_a
  @current_page = 1
  @count = @total_size = resultset.size
  resultset
end

#server_utc_timeObject



451
452
453
# File 'lib/sidekiq/web/helpers.rb', line 451

def server_utc_time
  Time.now.utc.strftime("%H:%M:%S UTC")
end

#singularize(str, count) ⇒ Object



115
116
117
118
119
120
121
# File 'lib/sidekiq/web/helpers.rb', line 115

def singularize(str, count)
  if count == 1 && str.respond_to?(:singularize) # rails
    str.singularize
  else
    str
  end
end

#sort_direction_labelObject



263
264
265
# File 'lib/sidekiq/web/helpers.rb', line 263

def sort_direction_label
  (url_params("direction") == "asc") ? "&uarr;" : "&darr;"
end

#sorted_processesObject

Sorts processes by hostname following the natural sort order



276
277
278
279
280
281
282
283
284
285
286
# File 'lib/sidekiq/web/helpers.rb', line 276

def sorted_processes
  @sorted_processes ||= begin
    return processes unless processes.all? { |p| p["hostname"] }

    processes.to_a.sort_by do |process|
      # Kudos to `shurikk` on StackOverflow
      # https://stackoverflow.com/a/15170063/575547
      process["hostname"].split(/(\d+)/).map { |a| /\d+/.match?(a) ? a.to_i : a }
    end
  end
end

#statsObject



288
289
290
# File 'lib/sidekiq/web/helpers.rb', line 288

def stats
  @stats ||= Sidekiq::Stats.new
end

#store_nameObject



10
11
12
13
14
15
# File 'lib/sidekiq/web/helpers.rb', line 10

def store_name
  hash = redis_info
  return "Dragonfly" if hash.has_key?("dragonfly_version")
  return "Valkey" if hash.has_key?("valkey_version")
  "Redis"
end

#store_versionObject



17
18
19
20
21
22
# File 'lib/sidekiq/web/helpers.rb', line 17

def store_version
  hash = redis_info
  return hash["dragonfly_version"] if hash.has_key?("dragonfly_version")
  return hash["valkey_version"] if hash.has_key?("valkey_version")
  hash["redis_version"]
end

#strings(lang) ⇒ Object



68
69
70
71
72
73
74
75
76
77
78
79
# File 'lib/sidekiq/web/helpers.rb', line 68

def strings(lang)
  @@strings ||= {}

  # Allow sidekiq-web extensions to add locale paths
  # so extensions can be localized
  @@strings[lang] ||= config.locales.each_with_object({}) do |path, global|
    find_locale_files(lang).each do |file|
      strs = parse_yaml_new(file)
      global.merge!(strs[lang])
    end
  end
end

#style_tag(location, **kwargs) ⇒ Object



24
25
26
27
28
29
30
31
32
33
34
35
36
37
# File 'lib/sidekiq/web/helpers.rb', line 24

def style_tag(location, **kwargs)
  global = location.match?(/:\/\//)
  location = root_path + location if !global && !location.start_with?(root_path)
  attrs = {
    type: "text/css",
    media: "screen",
    rel: "stylesheet",
    nonce: csp_nonce,
    href: location
  }
  add_to_head do
    html_tag(:link, attrs.merge(kwargs))
  end
end

#t(msg, options = {}) ⇒ Object



254
255
256
257
258
259
260
261
# File 'lib/sidekiq/web/helpers.rb', line 254

def t(msg, options = {})
  string = get_locale[msg] || strings("en")[msg] || msg
  if options.empty?
    string
  else
    string % options
  end
end

#text_directionObject



189
190
191
# File 'lib/sidekiq/web/helpers.rb', line 189

def text_direction
  get_locale["TextDirection"] || "ltr"
end

#to_display(arg) ⇒ Object



379
380
381
382
383
384
385
386
387
# File 'lib/sidekiq/web/helpers.rb', line 379

def to_display(arg)
  arg.inspect
rescue
  begin
    arg.to_s
  rescue => ex
    "Cannot display argument: [#{ex.class.name}] #{ex.message}"
  end
end

#to_json(x) ⇒ Object



111
112
113
# File 'lib/sidekiq/web/helpers.rb', line 111

def to_json(x)
  Sidekiq.dump_json(x)
end

#to_query_string(hash) ⇒ Object



348
349
350
351
352
# File 'lib/sidekiq/web/helpers.rb', line 348

def to_query_string(hash)
  hash.map { |key, value|
    SAFE_QPARAMS.include?(key) ? "#{key}=#{CGI.escape(value.to_s)}" : next
  }.compact.join("&")
end

#truncate(text, truncate_after_chars = 2000) ⇒ Object



354
355
356
# File 'lib/sidekiq/web/helpers.rb', line 354

def truncate(text, truncate_after_chars = 2000)
  (truncate_after_chars && text.size > truncate_after_chars) ? "#{text[0..truncate_after_chars]}..." : text
end

#unfiltered?Boolean

sidekiq/sidekiq#3243

Returns:

  • (Boolean)


245
246
247
248
# File 'lib/sidekiq/web/helpers.rb', line 245

def unfiltered?
  s = url_params("substr")
  yield unless s && s.size > 0
end

#user_preferred_languagesObject

See www.rfc-editor.org/rfc/rfc9110.html#section-12.5.4 Returns an array of language tags ordered by their quality value

Inspiration taken from github.com/iain/http_accept_language/blob/master/lib/http_accept_language/parser.rb



201
202
203
204
205
206
207
208
209
210
211
# File 'lib/sidekiq/web/helpers.rb', line 201

def user_preferred_languages
  languages = env["HTTP_ACCEPT_LANGUAGE"]
  languages.to_s.gsub(/\s+/, "").split(",").map { |language|
    locale, quality = language.split(";q=", 2)
    locale = nil if locale == "*" # Ignore wildcards
    quality = quality ? quality.to_f : 1.0
    [locale, quality]
  }.sort { |(_, left), (_, right)|
    right <=> left
  }.map(&:first).compact
end

#worksetObject



267
268
269
# File 'lib/sidekiq/web/helpers.rb', line 267

def workset
  @work ||= Sidekiq::WorkSet.new
end